﻿
;Vous pouvez consulter les fichiers langues de l'EDI dans le répertoire catalogs de PureBasic
              
; -----------------------------------------------------------------
;  Exemple d'une gestion simple des langues pour une application
; -----------------------------------------------------------------
;
; Avantage de cette solution :
;  - Les chaînes de caractères sont identifiées par un groupe et un nom clé, ce qui permet  
;    de mieux les organiser avec leurs propres descriptions et de développer plus facilement.
;
;  - Les chaînes de caractères sont indexées et triées lors de leur chargement,
;    ce qui permet un accès rapide malgré qu'elles soient appelées par leur nom.
;
;  - Une langue par défaut est définie dans le code (DataSection), aussi même 
;     si un fichier externe de langue est manquant ou périmé, il y a toujours la langue
;     par défaut présente.
;
;  - La liste de mots est facile à élargir. Il suffit d'ajouter une nouvelle entrée
;     dans la DataSection ainsi que le fichier de langue et d'utiliser le nouveau groupe et nom clé.
;
;  - Les fichiers de langues additionnels sont dans le format "PB preference" qui
;    permet de les maintenir facilement.
;
; Utilisation:
;  - définissez le langage par défaut dans la DataSection comme montré plus bas
;  - Utilisez LoadLanguage() au départ pour charger le langage par défaut ou le fichier externe.
;  - Utilisez Language(Group$, Name$) pour accéder aux mots des langages.
;
; -----------------------------------------------------------------
 
; Divers
;
Global NbLanguageGroups, NbLanguageStrings
 
Structure LanguageGroup
  Name$
  GroupStart.l
  GroupEnd.l
  IndexTable.l[256]
EndStructure
 
; Cette procédure charge la langue depuis un fichier, ou le langage par défaut.
; Elle doit être appelée au moins une fois avant d'utiliser n'importe quel mot.
;
; Cette procédure permet de :
; - charger et trier le langage par défaut fourni
; - charger n'importe quel langage depuis un fichier
;
; Cette méthode vous permet d'obtenir un mot d'une langue, même si le fichier n'est pas 
; trouvé, ou qu'un mot référencé par un index est introuvable dans le fichier.
; Vous récupererez la langue par défaut en utilisant la commande Language().
;
; Cette fonction peut être appelée plusieurs fois pour changer la langue utilisée durant l'exécution.
;
Procedure LoadLanguage(FileName$ = "")
 
  ; Fait d'abord un rapide compte de la DataSection
  ;
  NbLanguageGroups = 0
  NbLanguageStrings = 0
 
  Restore Language
  Repeat
 
    Read.s Name$
    Read.s String$
 
    Name$ = UCase(Name$)
 
    If Name$ = "_GROUP_"
      NbLanguageGroups + 1
    ElseIf Name$ = "_END_"
      Break
    Else
      NbLanguageStrings + 1
    EndIf
    
  ForEver
    
  Global Dim LanguageGroups.LanguageGroup(NbLanguageGroups)  ; ils sont tous là
  Global Dim LanguageStrings.s(NbLanguageStrings)
  Global Dim LanguageNames.s(NbLanguageStrings)
 
  ; On charge la langue standard (par défaut)
  ;  
  Group = 0
  StringIndex = 0  
 
  Restore Language
  Repeat
 
    Read.s Name$
    Read.s String$
 
    Name$ = UCase(Name$)
 
    If Name$ = "_GROUP_"
      LanguageGroups(Group)\GroupEnd   = StringIndex ; Mémorise le dernier index de chaînes associées à ce groupe
      Group + 1  
 
      LanguageGroups(Group)\Name$      = UCase(String$)  ; Mémorise le nom du groupe
      LanguageGroups(Group)\GroupStart = StringIndex + 1 ; Mémorise le premier index de chaînes associées à ce groupe
      For i = 0 To 255
        LanguageGroups(Group)\IndexTable[i] = 0
      Next i
      
    ElseIf Name$ = "_END_"
      Break
 
    Else
      StringIndex + 1 ; Incrémente l'index sur les chaînes
      LanguageNames(StringIndex)   = Name$ + Chr(1) + String$  ; Garde le nom de l'index et la valeur ensemble pour trier plus facilement
 
    EndIf
    
  ForEver
 
  LanguageGroups(Group)\GroupEnd   = StringIndex ; Configure le dernier index pour le dernier groupe !
  
  ; On effectue le tri et l'indexation pour chaque groupe
  ;
  For Group = 1 To NbLanguageGroups
    If LanguageGroups(Group)\GroupStart <= LanguageGroups(Group)\GroupEnd  
      
      SortArray(LanguageNames(), 0, LanguageGroups(Group)\GroupStart, LanguageGroups(Group)\GroupEnd)
  
      char = 0
      For StringIndex = LanguageGroups(Group)\GroupStart To LanguageGroups(Group)\GroupEnd
        LanguageStrings(StringIndex) = StringField(LanguageNames(StringIndex), 2, Chr(1)) ; Sépare la valeur de l'index
        LanguageNames(StringIndex)   = StringField(LanguageNames(StringIndex), 1, Chr(1))
 
        If Asc(Left(LanguageNames(StringIndex), 1)) <> char
          char = Asc(Left(LanguageNames(StringIndex), 1))
          LanguageGroups(Group)\IndexTable[char] = StringIndex
        EndIf
      Next StringIndex
      
    EndIf
  Next Group
 
  ; On essaye de chargé un fichier de langue externe.
  ;        
  If FileName$ <> ""
      
    If OpenPreferences(FileName$)
      For Group = 1 To NbLanguageGroups
        If LanguageGroups(Group)\GroupStart <= LanguageGroups(Group)\GroupEnd  
          PreferenceGroup(LanguageGroups(Group)\Name$)
          
          For StringIndex = LanguageGroups(Group)\GroupStart To LanguageGroups(Group)\GroupEnd
            LanguageStrings(StringIndex) = ReadPreferenceString(LanguageNames(StringIndex), LanguageStrings(StringIndex))
          Next StringIndex
        EndIf
      Next Group
      ClosePreferences()    
      
      ProcedureReturn #True
    EndIf    
 
  EndIf
  
  ProcedureReturn #True
EndProcedure
 
 
; Cette fonction retourne un mot pour la langue en cours d'utilisation.
; Chaque mot est identifié par un groupe et un nom (les deux sont insensibles à la casse)
;
; Si le mot n'est pas trouvé (ou non inclus dans la langue par défaut) le
; retour sera "##### String not found! #####". Cela aide à trouver les erreurs dans le code de la langue facilement
;
Procedure.s Language(Group$, Name$)
  Static Group.l  ; Pour un accès plus rapide quand on utilise le meme nom de groupe plusieurs fois
  Protected String$, StringIndex, Result
 
  Group$  = UCase(Group$)
  Name$   = UCase(Name$)
  String$ = "##### String not found! #####"  ; Pour aider à trouver les erreurs
 
  If LanguageGroups(Group)\Name$ <> Group$  ; Contrôle si c'est le même groupe à chaque fois
    For Group = 1 To NbLanguageGroups
      If Group$ = LanguageGroups(Group)\Name$
        Break
      EndIf
    Next Group
 
    If Group > NbLanguageGroups  ; Le groupe demandé n'est pas trouvé
      Group = 0
    EndIf
  EndIf
  
  If Group <> 0
    StringIndex = LanguageGroups(Group)\IndexTable[ Asc(Left(Name$, 1)) ]
    If StringIndex <> 0
 
      Repeat
        Result = CompareMemoryString(@Name$, @LanguageNames(StringIndex))
 
        If Result = #PB_String_Equal
          String$ = LanguageStrings(StringIndex)
          Break
 
        ElseIf Result = -1 ; Mot non trouvé
          Break
 
        EndIf
 
        StringIndex + 1
      Until StringIndex > LanguageGroups(Group)\GroupEnd
 
    EndIf
 
  EndIf
 
  ProcedureReturn String$
EndProcedure
 
 
DataSection
 
  ; C'es ici que la langue par défaut est définie.C'est une liste de Groupe, 
  ; Name avec quelques mot clefs spéciaux pour un groupe
  ;
  ; "_GROUP_" indique un nouveau gorupe dans la DataSection, la seconde valeur est le nom du groupe
  ; "_END_" indique la fin de la liste de la langue (comme il n'y a pas de nombre de mots prédéfinis)
  ;
  ; Note: Les index de mot sont insensibles à la casse pour se faciliter la vie.
  
  Language:
 
    ; ===================================================
    Data$ "_GROUP_",            "MenuTitle"
    ; ===================================================
 
      Data$ "File",             "File"
      Data$ "Edit",             "Edit"
                        
    ; ===================================================
    Data$ "_GROUP_",            "MenuItem"
    ; ===================================================
 
      Data$ "New",              "New"
      Data$ "Open",             "Open..."
      Data$ "Save",             "Save"
          
    ; ===================================================
    Data$ "_END_",              ""
    ; ===================================================
 
EndDataSection

; -----------------------------------------------------------------
; Exemple:
; -----------------------------------------------------------------
 
LoadLanguage()                ; charge la langue par défaut
;LoadLanguage("german.prefs") ; décommentez pour charger le fichier de langue allemand
 
; Récupère quelques mot de la langue
;
Debug Language("MenuTitle", "Edit")
Debug Language("MenuItem", "Save")
 
; -----------------------------------------------------------------

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 93
; FirstLine = 58
; Folding = -
; EnableXP